from __future__ import annotations

import functools
import json
import os
import platform
import shutil
import subprocess
import sys
import tarfile
import urllib
import urllib.error
import urllib.request
import zipfile
from contextlib import AbstractContextManager, chdir
from pathlib import Path
from tempfile import TemporaryDirectory, TemporaryFile
from typing import IO, Protocol

from proton_mail_trash.utils import set_env


# Largely copied from https://github.com/pre-commit/pre-commit/blob/main/pre_commit/languages/golang.py


_ARCH_ALIASES = {
    "x86_64": "amd64",
    "i386": "386",
    "aarch64": "arm64",
    "armv8": "arm64",
    "armv7l": "armv6l",
}
_ARCH = platform.machine().lower()
_ARCH = _ARCH_ALIASES.get(_ARCH, _ARCH)


class ExtractAll(Protocol):
    def extractall(self, path: str) -> None: ...


if sys.platform == "win32":  # pragma: win32 cover
    _EXT = "zip"

    def _open_archive(bio: IO[bytes]) -> AbstractContextManager[ExtractAll]:
        return zipfile.ZipFile(bio)
else:  # pragma: win32 no cover
    _EXT = "tar.gz"

    def _open_archive(bio: IO[bytes]) -> AbstractContextManager[ExtractAll]:
        return tarfile.open(fileobj=bio)


@functools.lru_cache
def _infer_go_version() -> str:
    resp = urllib.request.urlopen("https://go.dev/dl/?mode=json")
    return json.load(resp)[0]["version"].removeprefix("go")  # type: ignore[no-any-return]


def _get_url() -> str:
    os_name = platform.system().lower()
    version = _infer_go_version()
    return f"https://dl.google.com/go/go{version}.{os_name}-{_ARCH}.{_EXT}"


def _install_go(dest: Path) -> None:
    try:
        resp = urllib.request.urlopen(_get_url())  # noqa: S310
    except urllib.error.HTTPError as e:  # pragma: no cover
        if e.code == 404:  # noqa: PLR2004
            msg = (
                f"Could not find a version matching your system requirements "
                f"(os={platform.system().lower()}; arch={_ARCH})"
            )
            raise ValueError(
                msg,
            ) from e
        raise
    else:
        with TemporaryFile() as f:
            shutil.copyfileobj(resp, f)
            f.seek(0)

            with _open_archive(f) as archive:
                archive.extractall(dest)  # type: ignore[arg-type]  # noqa: S202
        shutil.move(dest / "go", dest / ".go")


def build(repo_url: str, build_path: str, bin_name: str, bin_dest: Path) -> None:
    if shutil.which("git") is None:
        print(
            "ERROR: git not found. "
            "Please ensure git is installed and available on your PATH. "
            "You can install it here: https://git-scm.com/downloads"
        )
        sys.exit(1)

    with TemporaryDirectory() as tempdir_s, chdir(tempdir_s):
        tempdir = Path(tempdir_s)
        print("Downloading go...")
        _install_go(tempdir)
        go_root = tempdir / ".go"
        go_bin = go_root / "bin"

        print("Downloading git repo...")
        subprocess.run(["git", "clone", repo_url, "repo"], check=True)  # noqa: S603, S607
        env: dict[str, str] = {
            "GOPATH": str(tempdir),
            "GOTOOLCHAIN": "local",
            "GOROOT": str(go_root),
            "PATH": os.pathsep.join((str(go_bin), os.environ["PATH"])),
        }
        with chdir("repo"), set_env(env):
            print("Building...")
            subprocess.run(["go", "build", build_path], check=True)  # noqa: S603, S607
            shutil.move(bin_name, bin_dest)



from __future__ import annotations

import os
import subprocess
import sys
from pathlib import Path
from time import sleep

from proton_mail_trash import golang
from proton_mail_trash.utils import PopenContext


PATH = Path(__file__).parent / "hydroxide"
if sys.platform == "win32":
    PATH = PATH.with_suffix(".exe")
if sys.platform == "win32":
    CONFIG_PATH = Path(os.getenv("APPDATA"))
else:
    CONFIG_PATH = Path.home() / ".config"
AUTH_PATH = CONFIG_PATH / "hydroxide/auth.json"


def build() -> None:
    if (
        input(
            "I will now bootstrap hydroxide. I only have to do this once. "
            "This is done in an isolated environment.\n"
            "Continue? (y/n) "
        )
        != "y"
    ):
        print("Abort.")
        sys.exit(1)

    bin_name = "hydroxide"
    if sys.platform == "win32":
        bin_name += ".exe"
    golang.build(
        "https://github.com/emersion/hydroxide", "./cmd/hydroxide", bin_name, PATH
    )


def get(user: str) -> Path:
    if not PATH.exists():
        build()
        assert PATH.exists()  # noqa: S101
    if not AUTH_PATH.exists():
        print("Authenticating via Hydroxide, you only have to do this once.")
        subprocess.run([PATH, "auth", user], check=True)  # noqa: S603
        assert AUTH_PATH.exists()  # noqa: S101
        print(
            "Authenticated! Please store the bridge password you got, "
            "you will be prompted for it on each run."
        )
    return PATH


def run_path(hydroxide: Path) -> PopenContext:
    p = PopenContext([hydroxide, "imap"])
    sleep(0.25)
    return p


def run(user: str) -> PopenContext:
    return run_path(get(user))



from __future__ import annotations

import sys
from datetime import date, timedelta
from email.parser import HeaderParser
from getpass import getpass
from imaplib import IMAP4

from proton_mail_trash import hydroxide


PORT = 1143


def main() -> None:
    user = input("E-mail: ")
    with hydroxide.run(user):
        password = getpass("Password: ")

        today = date.today()  # noqa: DTZ011 only using naive datetimes
        interval = timedelta(days=30)

        end = today - interval
        criteria = f"(BEFORE {end.strftime('%d-%b-%Y')})"

        box = IMAP4("localhost", PORT)
        box.login(user, password)
        box.select("Trash")
        _typ, data = box.search(None, criteria)
        uids = data[0].split()

        mailparser = HeaderParser()
        to_delete = []
        print("### The following messages will be permanently deleted:")
        for uid in uids:
            _resp, data = box.uid("fetch", uid, "(BODY[HEADER])")
            msg = mailparser.parsestr(data[0][1].decode())
            info = f"{msg['From']}, {msg['Date']}, {msg['Subject']}"
            info = info.replace("\n", "").replace("\r", "")
            print(info)
            to_delete.append(uid)

        if (
            input(
                "### The preceding messages will be permanently deleted. "
                "Continue? (y/n) "
            )
            != "y"
        ):
            print("Abort.")
            sys.exit(1)

        for uid in to_delete:
            print(uid.decode())
            box.store(uid, "+FLAGS", "\\Deleted")

        print("Expunging...")
        box.expunge()
        print("Closing...")
        box.close()
        print("Logging out...")
        box.logout()
        print("Stopping Hydroxide...")
